c-------------------------------------------------------------------------
c
      subroutine schsol(ruae,r,rab,sqr,a,b,v,z,ee,
     +  nnlz,wwnl,nkkk,snl,snlo,ncspvs,mesh,thresh,ifprt,idim1,idim2)
c
c     routine to compute non relativistic solutions to
c     schroedinger equation
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
c
c.....logarithmic radial mesh information
      dimension r(idim1),rab(idim1),sqr(idim1)
c.....potentials for the all electron calculation
      dimension ruae(idim1),v(idim1)
c.....wavefunctions
      dimension snl(idim1,idim2),snlo(idim1)
c.....shell labels, energies, occupancies and real space cutoff
      dimension nnlz(idim2),ee(idim2),wwnl(idim2),nkkk(idim2)
c.....scratch
      dimension yval(-1:1)
c
c.....variable for file = 0
      integer stderr
      common /files/ stderr,input,iout,ioae,iplot,iologd,iops
c
c--------------------------------------------------------------------------
c
c               r o u t i n e  i n i t i a l i s a t i o n
c
c     set the maximum number of iterations for improving wavefunctions
      itmax = 50
c
      do 10 j = 1,ncspvs
      do 10 i = 1,mesh
        snl(i,j) = 0.0d0
  10  continue
c
c--------------------------------------------------------------------------
c
c               m a i n  l o o p  o v e r  o r b i t a l s
c
      do 100 iorb = 1,ncspvs
c
c       check that this orbital is required
        if ( wwnl(iorb) .lt. 0.0d0 ) then
          ee(iorb) = 0.0d0
          nkkk(iorb) = 1
          snl(1,iorb) = 0.0d0
          goto 100
        endif
c
c       compute the quantum numbers
        ncur = nnlz(iorb) / 100
        lcur = nnlz(iorb) / 10 - ncur * 10
        llp = lcur * ( lcur + 1 )
c
c       set the starting energies for the iterative loop
        ecur = ee(iorb)
        eold = 0.0d0
        emax = + 1.0d0
        emin = - 1.0d+10
c
c-------------------------------------------------------------------------
c
c            i t e r a t i o n  o n  e i g e n f u n c t i o n s
c
        do 200 iter = 1,itmax
c
c         compute the sums of potential and centrifugal terms
c         required for the radial integrator
c
          call setv(r,rab,ruae,b,ecur,llp,v,mesh,idim1)
c
c----------------------------------------------------------------------------
c
c         c l a s s i c a l  t u r n i n g  a n d  i n f i n i t y
c
c         first attempt to locate the classical turning point
          do 210 i = mesh,2,-1
            if ( v(i) .lt. 0.0d0 ) goto 220
            if ( i .eq. 2 ) then
c             if the energy is reasonable this point should not be reached
              write(iout,*) '***error in subroutine schsol'
              write(iout,500) nnlz(iorb),iter,ecur,mesh
  500         format(' no tuning point fot state nnlz =',i4,
     +        ' on iteration',i3,/,' with ecur = ',f10.5,
     +        ' and v(mesh) = ',f10.5)
              call exit(1)
            endif
  210     continue
c
  220     continue
c
          if ( i .gt. mesh - 20 ) then
c           state unbound or close to being unbound
            nctp = mesh - 20
            ninf = mesh
          else
c           state well bound so i is nctp
            nctp = i
c           find the pracitcal infinity
            tolinf = 1.0d-18
            tolinf = dlog(tolinf) ** 2
            do 230 i = nctp,mesh
              alpha2 = v(i) * (dble(i-nctp)**2)
              if ( alpha2 .gt. tolinf ) goto 240
  230       continue
c
  240       continue
            ninf = i
          endif
c
          if ( ninf .eq. mesh ) then
            write(iout,*) '***warning in subroutine schsol'
            write(iout,540) nnlz(iorb),ecur,iter
  540       format(' state ',i4,' with energy',f9.5,
     +      ' either unbound or requires larger',/,
     +      ' radial mesh with energy on iteration',i4)
          endif
c
c         if ( ifprt .eq. 1 ) write(iout,520) nnlz(iorb),
c    +      iter,nctp,ninf
  520     format(' state nnlz=',i4,' on iteration',i3,' required',
     +    ' nctp =',i5,' and ninf =',i5)
c
c---------------------------------------------------------------------------
c
c               o u t w a r d  i n t e g r a t i o n 
c
c         compute the starting value of psi using taylor expansion 
          zz = 0.0d0
          vzero = ruae(2) / r(2)
          call orgsnl(r,snlo,1,3,lcur,zz,vzero,ecur,idim1)
c
c         convert to the phi function
          snlo(1) = snlo(1) / sqr(1)
          snlo(2) = snlo(2) / sqr(2)
          snlo(3) = snlo(3) / sqr(3)
c
c         call the integration routine
          call difsol(v,snlo,4,nctp,idim1)
c
          snlctp = snlo(nctp)
c
c--------------------------------------------------------------------------
c
c               i n w a r d  i n t e g r a t i o n
c
          if ( ninf .lt. mesh ) then
c           exponential start up of the wavefunction
            alpha = sqrt(v(ninf))
            snlo(ninf) = exp( - alpha * dble(ninf - nctp ) )
            alpha = sqrt(v(ninf-1))
            snlo(ninf-1) = exp ( - alpha * dble(ninf - nctp - 1) )
          else
c           linear start up
            snlo(ninf) = 0.0d0
            snlo(ninf-1) = r(ninf) - r(ninf-1)
          endif
c
          call difsol(v,snlo,ninf-2,nctp,idim1)
c
c--------------------------------------------------------------------------
c
c         c h e c k  t h a t  t h e  n u m b e r  o f  n o d e s  o k
c
c         renormalise snlo so continuous at nctp
          factor = snlctp / snlo(nctp)
          do 350 i = nctp,ninf
            snlo(i) = snlo(i) * factor
  350     continue
c
c         count the number of nodes
          call nodeno(snlo,2,ninf,nodes,idim1)
c
c--------------
c
          if ( nodes .lt. ncur - lcur - 1 ) then
c           energy is too low
            emin = ecur
            eold = ecur
            if ( ecur * 0.9d0 .gt. emax ) then
              ecur = 0.5d0 * ecur + 0.5d0 * emax 
            else
              ecur = 0.9d0 * ecur
            endif
            goto 200
          endif
c
          if ( nodes .gt. ncur - lcur - 1 ) then
c           energy is too high
            emax = ecur
            eold = ecur
            if ( ecur * 1.1d0 .lt. emin ) then
              ecur = 0.5d0 * ecur + 0.5d0 * emin
            else
              ecur = 1.1d0 * ecur
            endif
            goto 200
          endif
c
c------------------------------------------------------------------------------
c
c         v a r i a t i o n a l  i m p ro v e m e n t  o f  e c u r
c
c         find normalisation of psi using simpson's rule
          factor = 0.0d0
          xw = 4.0d0
          do 360 i = 2,ninf
            factor = factor + xw * rab(i) * (snlo(i)*sqr(i))**2
            xw = 6.0d0 - xw
  360     continue
          factor = factor / 3.0d0
c
c         compute the noumerov discontinuity in the phi function
          do 370 i = -1,1
            yval(i) = snlo(nctp+i) * ( 1.0d0 - v(nctp+i) / 12.0d0 )
  370     continue
          d2p = v(nctp) * snlctp
          disc = - yval(-1) + 2.0d0*yval(0) - yval(1) + d2p
c
c         variational estimate for the change in ecur
          decur = snlctp*disc*sqr(nctp)**2 / (factor*rab(nctp))
c
c         write(iout,*) 'ecur,decur',ecur,decur
          if ( abs(decur) .lt. thresh ) goto 400
c
c         possible modification of decur to prevent instabilities
          if (5.0d0*abs(decur).gt.-ecur) decur = sign(ecur,decur)/5.0d0
c
          eold = ecur
          ecur = ecur + decur
c
          if ( iter .eq. itmax ) then
c           eigenfunction has not converged in allowed number of iterations
            write(iout,*) '***error in subroutine schsol'
            write(iout,530) nnlz(iorb),ee(iorb),ecur
  530       format(' state',i4,' could not be converged.',/,
     +      ' starting energy for calculation was',f15.5,
     +      ' and end value =',f15.5)
            call exit(1)
          endif
c
c       close iteration loop
  200   continue
c
c-----------------------------------------------------------------------------
c
  400   continue
c
c       update the energy
        ee(iorb) = ecur
c       update the real-space cutoff array
        nkkk(iorb) = ninf
c
c       update the wavefunction array
        factor = 1.0d0 / sqrt( factor )
        do 410 i = 1,ninf
          snl(i,iorb) = snlo(i) * sqr(i) * factor
  410   continue
c
c     close loop over bands
  100 continue
c
      return
      end
c
c----------------------------------------------------------------------------
c
      subroutine difsol(g,p,jj1,jj2,idim1)
c
c---------------------------------------------------------------------------
c     routine for solving second order differential equation of the form
c
c         2
c        d p(x)
c        ------  = g(x) p(x)
c           2
c         dx
c
c     using the method due to noumerov.
c     method taken from g.w. pratt, phys. rev. 88 p.1217 (1952).
c
c     routine integrates from jj1 to jj2 and can cope with both cases
c     jj1 < jj2 and jj1 > jj2.  first two starting values of p must be
c     provided by the calling program.
c---------------------------------------------------------------------------
c
      implicit double precision (a-h,o-z)
c
      dimension g(idim1),p(idim1)
c
      integer stderr
      common /files/ stderr,input,iout,ioae,iplot,iologd,iops
c
c     i n i t i a l i s a t i o n
c
      r12 = 1.0d0 / 12.0d0
c
c     decide whether integrating from:
c            left to right ---> isgn = + 1
c        or  right to left ---> isgn = - 1
c
      isgn = ( jj2 - jj1 ) / iabs( jj2 - jj1 )
c
c     run some test just to be conservative
      if ( isgn .eq. + 1 ) then
        if ( jj1 .le. 2 .or. jj2 .gt. idim1 ) then
          write(iout,10) isgn,jj1,jj2,idim1
          call exit(1)
        endif
      elseif ( isgn .eq. - 1 ) then
        if ( jj1 .ge. ( idim1 - 1 ) .or. jj2 .lt. 1 ) then
          write(iout,10) isgn,jj1,jj2,idim1
          call exit(1)
        endif
      else
        write(iout,10) isgn,jj1,jj2,idim1
      endif
c
  10  format(' ***error in subroutine difsol',/,
     +' isgn =',i2,' jj1 =',i5,' jj2 =',i5,' idim1 =',i5,
     +' are not allowed')
c
c     initialise current second derivative of p (d2p), ycur and yold
c
      i = jj1 - isgn
      d2p = g(i) * p(i)
      ycur = ( 1.0d0 - r12 * g(i) ) * p(i)
      i = i - isgn
      yold = ( 1.0d0 - r12 * g(i) ) * p(i)
c
c
c     b e g i n  i n t e g r a t i o n  l o o p
c
      do 100 i = jj1,jj2,isgn
        ynew = 2.0d0 * ycur - yold + d2p
        p(i) = ynew / ( 1.0d0 - r12 * g(i) )
        d2p  = g(i) * p(i)
        yold = ycur
        ycur = ynew
100   continue
c
      return
      end
c
c---------------------------------------------------------------------------
c
      subroutine setv(r,rab,ruae,b,ecur,llp,v,mesh,idim1)
c
c     compute the sums of potential and centrifugal terms
c     required for the radial integrator
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
c.....logarithmic radial mesh information
      dimension r(idim1),rab(idim1)
c.....potentials for the all electron calculation
      dimension ruae(idim1),v(idim1)
c
c
      v(1) = 0.0d0
      do 100 i = 2,mesh
        v(i) = b * b / 4.0d0 + rab(i) * rab(i) *
     +    ( ruae(i) / r(i) + dble(llp) / ( r(i) * r(i) ) - ecur )
  100 continue
c
      return
      end
c
c-------------------------------------------------------------------------
c
      subroutine vhxc(ipass,ifpcor,ruexch,ruhar,reexch,rscore,
     +  rsvale,rsatom,rspsco,rstot,exfact,r,rab,
     +  xi,xj,fu,ehar,emvxc,mesh,idim1)
c
c     calculate hartree and exchange-correlation potentials
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
c
c.....logarithmic radial mesh information
      dimension r(idim1),rab(idim1)
c.....hartree, exchange and correlation potentials
      dimension ruhar(idim1),reexch(idim1),ruexch(idim1)
c.....charge densities
      dimension rscore(idim1),rsvale(idim1),rsatom(idim1),rspsco(idim1)
c.....scratch arrays
      dimension xi(idim1),xj(idim1),fu(idim1),rstot(idim1)
c
      integer stderr
      common /files/ stderr,input,iout,ioae,iplot,iologd,iops
c
      data pi    /3.141592654/
c
c     specify cut-off density below which polynomial form will be used
c         exc =     coef1 * rho +     coef2 * rho**2
c         uxc = 2 * coef1 * rho + 3 * coef2 * rho**2
c     and solve for coefficients such that exc and uxc match at rhoc
      rhoc=1.d-15
      call xctype(exfact,rhoc,uxcc,excc)
      coef1 = ( 3.d0 * excc - uxcc ) / rhoc
      coef2 = (-2.d0 * excc + uxcc ) / rhoc**2
c
c     calculate total charge density and exchange-correlation
      ruexch(1)=0.0d0
      reexch(1)=0.0d0
      do 100 i=2,mesh
c       charge depends on whether this is ae or pseudo calculation
        if ( ipass .eq. 1 ) then
          rsatom(i) = rscore(i) + rsvale(i)
        elseif ( ipass .eq. 2 ) then
          rsatom(i) = rsvale(i)
        endif
c
c       section of code added 12/19/91 to prevent problems of
c       pseudo-ion having incorrect charge
c
        rstot(i)=rsatom(i)
        if (ipass.eq.2.and.ifpcor.gt.0) rstot(i)=rsatom(i)+rspsco(i)
        rho=rstot(i)/(4.*pi*r(i)**2)
c
c       take measures to ensure that rho is greater than zero
        if ( rstot(i) .lt. 1.0d-30 ) then
c         check if charge density is negative
          if ( rstot(i) .lt. -1.0d-30 ) then
            write(iout,*) '***error subroutine vhxc'
            write(iout,*) '-----------negative charge density----------'
            write(iout,*) 'rstot =',rstot(i),' at r =',r(i)
            call exit(1)
          endif
        endif
c
        if (rho.gt.rhoc) then
          call xctype(exfact,rho,uxc,exc)
        else
          exc =     coef1 * rho +     coef2 * rho**2
          uxc = 2 * coef1 * rho + 3 * coef2 * rho**2
        endif
c
        ruexch(i)=r(i)*uxc
        reexch(i)=r(i)*exc
  100 continue
c     ruexch contains exchange-correlation potential
c     reexch contains exchange-correlation energy
c
c     calculate atomic coulomb potential
      xi(1)=0.
      xj(1)=0.
      do 110 i=2,mesh
        xi(i)=rsatom(i)
        xj(i)=rsatom(i)/r(i)
  110 continue
      call radin(mesh,xi,rab,xim,idim1)
      call radin(mesh,xj,rab,xjmesh,idim1)
c
      do 120 i=1,mesh
        rehar=xi(i)+r(i)*(xjmesh-xj(i))
        ruhar(i)=rehar*2.0
  120 continue
c
      fu(1)=0.
      do 130 i=2,mesh
        fu(i)=0.5*ruhar(i)/r(i)*rsatom(i)
  130 continue
      call radin(mesh,fu,rab,ehar,idim1)
      fu(1)=0.
      do 140 i=2,mesh
        fu(i)=( reexch(i)*rstot(i) - ruexch(i)*rsatom(i) )/r(i)
  140 continue
      call radin(mesh,fu,rab,emvxc,idim1)
c
      return
      end
c
c     ****************************************************************
      subroutine xctype(alphax,rho,uxc,exc)
c     ****************************************************************
c
c ---------------------------------------------------------------
      implicit double precision (a-h,o-z)
c ---------------------------------------------------------------
      integer stderr
      common /files/ stderr,input,iout,ioae,iplot,iologd,iops
c ---------------------------------------------------------------
c
c     subroutine to calculate exchange-correlation potential and energy
c
c     alphax : key to type of exchange-correlation
c        alphax =  0. -->  perdew-zunger param of ceperley-alder
c        alphax = -1. -->  wigner interpolation
c        alphax = -2. -->  hedin-lundquist
c        alphax = -3. -->  gunnarson-lundquist
c        alphax = -4. -->  no exchange or correlation at all
c        other values -->  slater x-alpha with alpha = alphax
c
c     rho = density in electrons per au**3 (input)
c
c     uxc = exchange-correlation potential in ry (output)
c     exc = exchange-correlation energy    in ry (output)
c
      data pi    /3.141592654/
      data third /0.333333333/
      data qq    /0.620350491/
      data cc    /0.916330586/
c
c     qq = (3./(4.*pi))**(1./3.)
c     cc=pi*(1.5/pi)**(5./3.)
c
c     perdew-zunger exchange-correlation
      data beta1 /1.0529/
      data beta2 /0.3334/
      data gamma /-0.2846/
      data pza /0.0622/
      data pzb /-0.096/
      data pzc /0.0040/
      data pzd /-0.0232/
c     wigner exchange-correlation
      data aa /0.88/
      data bb /7.8/
c     hedin-lundquist exchange-correlation
      data ah /21./
      data ch /0.045/
c     gunnarson-lundquist exchange-correlation
      data ag /11.4/
      data cg /0.0666/
c
      rs=qq/rho**third
      ex=-cc/rs
      ux=4.0*ex/3.0
c     ex and ux are exchange energy and potential respectively
c
      if (alphax.eq. 0.) then
c
c       perdew-zunger correlation
        if (rs.gt.1.) then
          dd=1.0+beta1*sqrt(rs)+beta2*rs
          dn=1.0+(7./6.)*beta1*sqrt(rs)+(4./3.)*beta2*rs
          ec=gamma/dd
          uc=ec*dn/dd
        else
          rl=log(rs)
          ec= pza*rl + pzb + pzc*rs*rl + pzd*rs
          uc= pza*rl + (pzb-pza/3.) + (2.*pzc/3.)*rs*rl
     1       + ((2*pzd-pzc)/3.)*rs
        endif
        uxc=ux+uc
        exc=ex+ec
c
      else if (alphax.eq.-1.) then
c
c       wigner correlation
        ec=-aa/(rs+bb)
        ecmuc=rs*aa/(3.0*(rs+bb)**2)
        uxc=ux+ec-ecmuc
        exc=ex+ec
c
      else if (alphax.eq.-2.) then
c
c       hedin-lundquist correlation
        xx=rs/ah
        al=log(1.+1./xx)
        uc=-ch*al
        ecmuc=-ch*(al*xx**3+xx/2.-xx*xx-1./3.)
        uxc=ux+uc
        exc=ex+uc+ecmuc
c
      else if (alphax.eq.-3.) then
c
c       gunnarson-lundquist correlation
        xx=rs/ag
        al=log(1.+1./xx)
        uc=-cg*al
        ecmuc=-cg*(al*xx**3+xx/2.-xx*xx-1./3.)
        uxc=ux+uc
        exc=ex+uc+ecmuc
c
      else if (alphax.eq.-4.) then
c
c       no exchange or correlation at all (for debugging only)
        uxc=0.
        exc=0.
c
      else if (alphax.gt. 0.) then
c
c       slater exchange-correlation
c       alphax = 2/3 corresponds to pure exchange
        uxc=ux * 1.5 * alphax
        exc=ex * 1.5 * alphax
c
      else
c
        write(iout,*) '***error in subroutine xctype'
        write(iout,*) 'alphax = ',alphax,' is out of range'
        call exit(1)
c
      endif
c
      return
      end
c
c----------------------------------------------------------------------------
c
      subroutine radin(mesh,func,rab,asum,idim1)
c
c     simpson's rule integrator for function stored on the
c     logarithmic mesh
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
c
c.....logarithmic radial mesh information
      dimension rab(idim1)
c.....function to be integrated
      dimension func(idim1)
c
c.....variable for file = 0
      integer stderr
      common /files/ stderr,input,iout,ioae,iplot,iologd,iops
c
c     routine assumes that mesh is an odd number so run check
      if ( mesh - ( mesh / 2 ) * 2 .ne. 1 ) then
        write(iout,*) '***error in subroutine radin'
        write(iout,*) 'routine assumes mesh is odd but mesh =',mesh
        call exit(1)
      endif
c
c     should really be no case where func(1) .ne. 0
      if ( abs(func(1)) .gt. 1.0d-10 ) then
        write(iout,*) '***error in subroutine radin'
        write(iout,*) 'func(1) is non-zero, func(1) =',func(1)
        call exit(1)
      endif
c
      asum = 0.0d0
      r12 = 1.0d0 / 12.0d0
      f3  = func(1) * rab(1) * r12
      func(1) = 0.0d0
c
      do 100 i = 2,mesh-1,2
        f1 = f3
        f2 = func(i) * rab(i) * r12
        f3 = func(i+1) * rab(i+1) * r12
        asum = asum + 5.0d0*f1 + 8.0d0*f2 - 1.0d0*f3
        func(i) = asum
        asum = asum - 1.0d0*f1 + 8.0d0*f2 + 5.0d0*f3
        func(i+1) = asum
100   continue
c
      return
      end
c
c-------------------------------------------------------------------------
c
      subroutine rsae(snl,wwnl,nkkk,nnlz,rscore,rsvale,ncores,ncspvs,
     +  mesh,alpha,nbl,nbl0,rab,ipass,keyps,kkbeta,qfunc,dum,
     +  idim1,idim2,idim3,idim5)
c
c     compute the charge density for the all electron calculation
c     and compute valence corrections for generalized eigenvalue case
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
c.....logarithmic radial mesh information
      dimension rab(idim1)
c.....charge densities
      dimension rscore(idim1),rsvale(idim1)
c.....wavefunctions
      dimension snl(idim1,idim2)
c.....shell labels, energies, occupancies and real-space cutoff index
      dimension nnlz(idim2),wwnl(idim2),nkkk(idim2)
c.....q function
      dimension qfunc(idim1,idim3,idim3)
c.....work space for solution to inhomogeneous equation and overlaps
      dimension alpha(idim3,idim2)
c.....indexing for the beta functions
      dimension nbl0(idim5),nbl(idim5)
c.....scratch
      dimension dum(idim1)
c
      integer stderr
      common /files/ stderr,input,iout,ioae,iplot,iologd,iops
c
c--------------------------------------------------------------------------
c
c     i n i t i a l i s a t i o n
c
c     zero the core and valence charge density arrays
      do 10 i = 1,mesh
        rscore(i) = 0.0d0
        rsvale(i) = 0.0d0
   10 continue
c
c--------------------------------------------------------------------------
c
c     c o r e  c h a r g e  d e n s i t y  (  o n l y  i f  a e  )
c
      if ( ipass .eq. 1 ) then
c
        do 100 j = 1,ncores
        do 100 i = 1,nkkk(j)
          rscore(i) = rscore(i) + wwnl(j) * snl(i,j)**2
  100   continue
c
      endif
c
c--------------------------------------------------------------------------
c
c            v a l e n c e  c h a r g e  d e n s i t y
c
      do 110 j = ncores+1,ncspvs
c
        do 120 i = 1,nkkk(j)
          rsvale(i) = rsvale(i) + wwnl(j) * snl(i,j)**2
  120   continue
c
c       possible extra contribution from vanderbilt scheme
        if ( ipass .eq. 2 .and. keyps .eq. 3 ) then
c
          ncur = nnlz(j) / 100
          lcur = nnlz(j) / 10 - 10 * ncur
          lp   = lcur + 1
c
          do 210 i = 1,mesh
            dum(i) = rsvale(i)
  210     continue
          call radin(mesh,dum,rab,asum,idim1)
          write(iout,*) 'intermed rsvale for lp=',lp,asum
          write(iout,*) 'alpha=',(alpha(nbl0(lp)+ib,j),ib=1,nbl(lp))
c
          do 220 ib = 1,nbl(lp)
          do 220 jb = 1,nbl(lp)
            ibeta = nbl0(lp) + ib
            jbeta = nbl0(lp) + jb
            fac = wwnl(j) * alpha(ibeta,j) * alpha(jbeta,j)
            do 230 i = 1,kkbeta
              rsvale(i) = rsvale(i) + fac * qfunc(i,ibeta,jbeta)
  230       continue
  220     continue
c
          do 240 i = 1,mesh
            dum(i) = rsvale(i)
  240     continue
          call radin(mesh,dum,rab,asum,idim1)
          write(iout,*) 'integrated rsvale after lp',lp,asum
c
        endif
c
c     close loop over valence orbitals
  110 continue
c
c-----------------------------------------------------------------------
c
c         b a l a n c e  v a l e n c e  c h a r g e
c
      if ( ipass .eq. 2 .and. keyps .eq. 3 ) then
c
        wwsum = 0.0d0
        do 300 j = ncores+1,ncspvs
          wwsum = wwsum + wwnl(j)
  300   continue
c
        do 310 i = 1,mesh
          dum(i) = rsvale(i)
  310   continue
c
        call radin(mesh,dum,rab,asum,idim1)
c
        write(iout,*) 'balancing valence density, asum =',asum,
     +    ' wwsum =',wwsum
c
        if ( abs( asum - wwsum ) .gt. 8.0d-05 ) then
          write(iout,*) '***error in subroutine rsae'
          write(iout,*) 'asum and wwsum are not consistent'
          call exit(1)
        endif
c
        if ( abs(asum) .lt. 1.0d-05 ) return
c
        fac = wwsum / asum
        do 320 i = 1,mesh
          rsvale(i) = fac * rsvale(i)
  320   continue
c
      endif
c
      return
      end
c
c------------------------------------------------------------------------
c
      subroutine mixer(ruae,runuc,ruhar,ruexch,rudif,damp,
     +  tol,delta,iselfc,mesh,idim1)
c
c     routine for mixing old and new charge potentials.
c     if the potentials are found to be self-consistent iselfc = +1
c                                             otherwise iselfc = -1
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
c.....potentials for the all electron calculation
      dimension ruae(idim1),runuc(idim1),rudif(idim1)
c.....hartree, exchange and correlation potentials
      dimension ruhar(idim1),ruexch(idim1)
c
c     initialisation
      iselfc = - 1
      delta = 0.0d0
c
c     compute the difference between new and old all electron potentials
      do 100 i = 2,mesh
        rudif(i) = runuc(i) + ruhar(i) + ruexch(i) - ruae(i)
        if ( delta .lt. abs( rudif(i) ) ) delta = abs( rudif(i) )
  100 continue
c
c     check whether the potential is self consistent
      if ( delta .lt. tol ) then
        iselfc = + 1
c       on last iteration, update potential without damping
        do 190 i = 2,mesh
          ruae(i) = ruae(i) + rudif(i)
  190   continue
      else
        do 200 i = 2,mesh
          ruae(i) = ruae(i) + damp * rudif(i)
  200   continue
      endif
c
      return
      end
c
c----------------------------------------------------------------------------
c
      subroutine orgsnl(r,snlo,j1,j2,lcur,zz,vzero,ecur,idim1)
c
c     generate an analytic starting guess for the wavefunction
c     at the origin using taylor series expansion.
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
c
c.....logarithmic radial mesh information
      dimension r(idim1)
c.....wavefunctions
      dimension snlo(idim1)
c
c
c     compute the starting value of psi according to
c
c     psi(r) = r**(l+1) * ( 1 + aa * r + bb * r**2 + ... )
c
c     where:
c           aa = - z / (l+1) 
c           bb = ( 2 * z * z / (l+1) + v(0) - e ) / (4*l+6)
c
c
      lp = lcur + 1
      aa = - zz / dble(lp)
      bb = ( 2 * zz * zz / dble(lp) + vzero - ecur ) / dble(4*lcur+6)
c
      do 100 i = j1,j2
        snlo(i) = (r(i)**lp) * ( 1.0d0 + aa*r(i) + bb*r(i)*r(i) )
100   continue
c
      return
      end
c
c----------------------------------------------------------------------------
c
c     ***************************************************************
      subroutine lderiv(ifprt,ipass,keyps,ilogd,klogd,irel,emax,emin,
     +  nnt,zz,vzero,b,r,rab,sqr,ruae,v,snlo,yy,flname,mesh,
     +  vloc,beta,nbl,nbl0,gamma,alpha,eta,kkbeta,ddd,qqq,
     +  bndmat,idim1,idim3,idim5,idim7)
c     ***************************************************************
c     compute logarithmic derivatives over an energy range and
c       print comparisons between ae and pseudo case
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
      parameter ( idim10 = 81 , idim11 = 10 )
c
c.....logarithmic radial mesh information
      dimension r(idim1),rab(idim1),sqr(idim1)
c.....potentials for the all electron calculation
      dimension ruae(idim1),v(idim1)
c.....wavefunctions
      dimension snlo(idim1),yy(idim1,2)
c.....beta and q functions and q pseudization coefficients
      dimension beta(idim1,idim3)
c.....angular momenta, reference energy, qij and dij of vanderbilt scheme
      dimension qqq(idim3,idim3),ddd(idim3,idim3)
c.....work space for solution to inhomogeneous equation and overlaps
      dimension eta(idim1,idim7),alpha(idim3)
c.....indexing for the beta functions
      dimension nbl0(idim5),nbl(idim5)
c.....local component of the pseudopotential
      dimension vloc(idim1)
c.....scratch arrays
      dimension gamma(idim1),bndmat(idim1,2,2)
c.....logarithmic derivative arrays
      dimension dlsav(idim10,idim11),ltab(idim11)
c.....symbols for the line printer plot out
      dimension isymbl(9)
c.....file name array
      character*40 flname(6)
c
c     initialise the symbol array
      data  isymbl / 1h!,1h1,1h2,1h3,1h4,1h5,1h6,1h7,1h8 /
c
      integer stderr
      common /files/ stderr,input,iout,ioae,iplot,iologd,iops
c
      save isymbl, dlsav, ltab
c
c----------------------------------------------------------------------------
c
c                    i n i t i a l i s a t i o n
c
      write (iout,10) ipass
   10 format ('0entering subroutine lderiv, pass ',i1)
c
      nnt1 = nnt + 1
c
      if ( ipass .lt. 1 .or. ipass .gt. 2 ) then
        write(iout,*) '***error in subroutine lderiv'
        write(iout,*) 'ipass =',ipass,' is out of range'
        call exit(1)
      endif
c
      if ( ipass .eq. 1 ) then
c
c       check that idim10 is sufficient
        if ( nnt1 .gt. idim10 ) then
          write(iout,*) '***error in subroutine lderiv'
          write(iout,*) 'nnt+1=',nnt1,' but idim10=',idim10
          call exit(1)
        endif
c
c       check that idim11 is sufficient
        if ( 2 * ilogd + 2 .gt. idim11 ) then
          write(iout,*) '***error in subroutine lderiv'
          write(iout,*) '2*ilogd+2=',2*ilogd+2,' but idim11',idim11
          call exit(1)
        endif
c
c       open the file for storing the logarithmic derivative information
        open( unit = iologd , file = flname(5) , status = 'unknown' ,
     +    form = 'unformatted' )
c
c       zeroing
        do 20 j = 1,2*ilogd+2
        do 20 i = 1,nnt1
          dlsav(i,j) = 0.0d0
   20   continue
c
      endif
c
c---------------------------------------------------------------------------
c
c          c o m p u t e  t h e  l o g  d e r i v a t i v e s
c
      dele = ( emax - emin ) / dble(nnt)
c
c     loop over the angular momenta
      do 100 lcur = 0,ilogd-1
c
        index = 3 + ilogd * ( ipass - 1 ) + lcur
        ltab(index) = lcur
c
        do 110 i = 1,nnt1
c
          ecur = emin + dele * dble(i-1)
c
          if ( ipass .eq. 1 ) then
c
            if ( irel .eq. 0 ) then
c
c             schroedinger equation is required
              call phase(zz,vzero,ecur,lcur,b,r,rab,sqr,
     +          ruae,v,snlo,mesh,klogd,dlwf,idim1)
c
            elseif ( irel .eq. 2 ) then
c
c             koelling and harmon scalar relativistic equation
              call phakoe(ruae,r,rab,bndmat,ecur,lcur,
     +          yy,v,klogd,dlwf,idim1)
c
            else
c
              write(iout,*) '***error in subroutine lderiv'
              write(iout,*) 'irel =',irel,' is not programmed'
              call exit(1)
c
            endif
c
c
          elseif ( ipass .eq. 2 .and. keyps .eq. 3 ) then
c
            call schgef(ecur,lcur,b,r,rab,sqr,vloc,v,
     +        snlo,mesh,klogd,dlwf,beta,nbl,nbl0,gamma,alpha,
     +        eta,kkbeta,ddd,qqq,idim1,idim3,idim5,idim7)
c
          else
            write(iout,*) '***error in subroutine lderiv'
            write(iout,*) 'ipass=',ipass,'and keyps=',keyps,'illegal'
            call exit(1)
          endif
c
          dlsav(i,index) = dlwf
c
  110   continue
c
        write (iologd) ilogd,nnt1,emin,dele,(dlsav(i,index),i=1,nnt1)
c
  100 continue
c
      if (ipass.eq.1.or.ifprt.lt.-2) return
c
c---------------------------------------------------------------------------
c
c     p r i n t  o u t  o f  l o g  d e r i v a t i v e s
c
      ndl = 2 + ilogd * ipass
      write (iout,200) (ltab(m),m=3,ndl)
  200 format (1h1,20x,23hlogarithmic derivatives//4x,3he  ,
     1   7(8x,2hl=,i1,5x))
      write (iout,210)
  210 format (1h0)
      do 220 i=1,nnt1
        e=emin+dele*dble(i-1)
        dlsav(i,1)=e
        dlsav(i,2)=0.
        write (iout,230) e,(dlsav(i,m),m=3,ndl)
  220 continue
  230 format (1x,f6.3,7(3x,e13.5))
c
c     line printer plot out of results
      call lpplot(8,dlsav,nnt1,ndl,nnt1,0,idim10,idim11,isymbl,-4.,4.)
c
      return
      end
c
c----------------------------------------------------------------------------
c
      subroutine phase(zz,vzero,ecur,lcur,b,r,rab,sqr,
     +  ruae,v,snlo,mesh,klogd,dlwf,idim1)
c
c     routine for computing logarithmic derivatives
c
c     output in dlwf is d ln ( r*psi(r) ) / dr
c
c----------------------------------------------------------------------------
c
      implicit double precision(a-h,o-z)
c
c
c.....logarithmic radial mesh information
      dimension r(idim1),rab(idim1),sqr(idim1)
c.....potentials for the all electron calculation
      dimension ruae(idim1)
c.....wavefunctions
      dimension snlo(idim1)
c.....scratch arrays
      dimension v(idim1)
c
c     compute the potential for this angular momentum and energy
      llp = lcur * ( lcur + 1 )
      call setv(r,rab,ruae,b,ecur,llp,v,mesh,idim1)
c
c     analytic starting value for snlo using taylor expansion
      call orgsnl(r,snlo,1,3,lcur,zz,vzero,ecur,idim1)
c
c     convert to the phi function
      snlo(1) = snlo(1) / sqr(1)
      snlo(2) = snlo(2) / sqr(2)
      snlo(3) = snlo(3) / sqr(3)
c
c     integrate the schroedinger equation outwards
      call difsol(v,snlo,4,klogd,idim1)
c
c     compute the logarithmic derivative
c
      snlom = snlo(klogd-1) * sqr(klogd-1)
      snlop = snlo(klogd)   * sqr(klogd)
c
      dwf =   snlop - snlom
      wf  = ( snlop + snlom ) * 0.5d0
      dr  =   r(klogd) -    r(klogd-1)
c
      dlwf = dwf / ( dr * wf )
c
      return
      end
c
c----------------------------------------------------------------------------
c
c     *******************************************
      subroutine nodeno(snlo,jj1,jj2,nodes,idim1)
c     *******************************************
c
c     routine counts the number of nodes of the wavefunction snlo
c     between the points jj1 and jj2
c
      implicit double precision (a-h,o-z)
c
c.....wavefunction array
      dimension snlo(idim1)
c
      nodes = 0
c
      do 100 i = jj1+1,jj2
        if ( snlo(i-1) * snlo(i) .lt. 0.0d0 ) nodes = nodes + 1
  100 continue
c
      return
      end
c
c---------------------------------------------------------------------------
